@@ -406,11 +406,11 @@ template<typename MetricType, typename StatisticType, typename MatType>
 template<typename Archive>
 Octree<MetricType, StatisticType, MatType>::Octree(
     Archive& ar,
-    const typename boost::enable_if<typename Archive::is_loading>::type*) :
+    const typename std::enable_if_t<Archive::is_loading::value>*) :
     Octree() // Create an empty tree.
 {
   // De-serialize the tree into this object.
-  ar >> data::CreateNVP(*this, "tree");
+  ar >> BOOST_SERIALIZATION_NVP(*this);
 }
 
 template<typename MetricType, typename StatisticType, typename MatType>
@@ -436,7 +436,7 @@ template<typename MetricType, typename StatisticType, typename MatType>
 template<typename VecType>
 size_t Octree<MetricType, StatisticType, MatType>::GetNearestChild(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType>>::type*) const
+    typename std::enable_if_t<IsVector<VecType>::value>*) const
 {
   // It's possible that this could be improved by caching which children we have
   // and which we don't, but for now this is just a brute force search.
@@ -459,7 +459,7 @@ template<typename MetricType, typename StatisticType, typename MatType>
 template<typename VecType>
 size_t Octree<MetricType, StatisticType, MatType>::GetFurthestChild(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType>>::type*) const
+    typename std::enable_if_t<IsVector<VecType>::value>*) const
 {
   // It's possible that this could be improved by caching which children we have
   // and which we don't, but for now this is just a brute force search.
@@ -600,7 +600,7 @@ template<typename VecType>
 typename Octree<MetricType, StatisticType, MatType>::ElemType
 Octree<MetricType, StatisticType, MatType>::MinDistance(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType>>::type*) const
+    typename std::enable_if_t<IsVector<VecType>::value>*) const
 {
   return bound.MinDistance(point);
 }
@@ -610,7 +610,7 @@ template<typename VecType>
 typename Octree<MetricType, StatisticType, MatType>::ElemType
 Octree<MetricType, StatisticType, MatType>::MaxDistance(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType>>::type*) const
+    typename std::enable_if_t<IsVector<VecType>::value>*) const
 {
   return bound.MaxDistance(point);
 }
@@ -621,7 +621,7 @@ template<typename VecType>
 math::RangeType<typename Octree<MetricType, StatisticType, MatType>::ElemType>
 Octree<MetricType, StatisticType, MatType>::RangeDistance(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType>>::type*) const
+    typename std::enable_if_t<IsVector<VecType>::value>*) const
 {
   return bound.RangeDistance(point);
 }
@@ -629,12 +629,10 @@ Octree<MetricType, StatisticType, MatType>::RangeDistance(
 //! Serialize the tree.
 template<typename MetricType, typename StatisticType, typename MatType>
 template<typename Archive>
-void Octree<MetricType, StatisticType, MatType>::Serialize(
+void Octree<MetricType, StatisticType, MatType>::serialize(
     Archive& ar,
     const unsigned int /* version */)
 {
-  using data::CreateNVP;
-
   // If we're loading and we have children, they need to be deleted.
   if (Archive::is_loading::value)
   {
@@ -644,62 +642,25 @@ void Octree<MetricType, StatisticType, MatType>::Serialize(
 
     if (!parent)
       delete dataset;
+
+    parent = NULL;
   }
 
-  ar & CreateNVP(begin, "begin");
-  ar & CreateNVP(count, "count");
-  ar & CreateNVP(bound, "bound");
-  ar & CreateNVP(stat, "stat");
-  ar & CreateNVP(parentDistance, "parentDistance");
-  ar & CreateNVP(furthestDescendantDistance, "furthestDescendantDistance");
-  ar & CreateNVP(metric, "metric");
+  ar & BOOST_SERIALIZATION_NVP(begin);
+  ar & BOOST_SERIALIZATION_NVP(count);
+  ar & BOOST_SERIALIZATION_NVP(bound);
+  ar & BOOST_SERIALIZATION_NVP(stat);
+  ar & BOOST_SERIALIZATION_NVP(parentDistance);
+  ar & BOOST_SERIALIZATION_NVP(furthestDescendantDistance);
+  ar & BOOST_SERIALIZATION_NVP(metric);
+  ar & BOOST_SERIALIZATION_NVP(dataset);
 
-  // Due to quirks of boost::serialization, depending on how the user
-  // serializes the tree, it's possible that the root of the tree will
-  // accidentally be serialized twice.  So if we are a first-level child, we
-  // avoid serializing the parent.  The true (non-duplicated) parent will fix
-  // the parent link.
-  bool hasFakeParent = false;
-  if (Archive::is_saving::value && parent != NULL && parent->parent == NULL)
-  {
-    Octree* fakeParent = NULL;
-    hasFakeParent = true;
-    ar & CreateNVP(fakeParent, "parent");
-    ar & CreateNVP(hasFakeParent, "hasFakeParent");
-  }
-  else
-  {
-    ar & CreateNVP(parent, "parent");
-    ar & CreateNVP(hasFakeParent, "hasFakeParent");
-  }
+  ar & BOOST_SERIALIZATION_NVP(children);
 
-  // Only serialize the dataset if we don't have a fake parent.  Otherwise, the
-  // real parent will come and set it later.
-  if (!hasFakeParent)
-    ar & CreateNVP(dataset, "dataset");
-
-  size_t numChildren = 0;
-  if (Archive::is_saving::value)
-    numChildren = children.size();
-  ar & CreateNVP(numChildren, "numChildren");
   if (Archive::is_loading::value)
-    children.resize(numChildren);
-
-  for (size_t i = 0; i < numChildren; ++i)
-  {
-    std::ostringstream oss;
-    oss << "child" << i;
-    ar & CreateNVP(children[i], oss.str());
-  }
-
-  // Fix the child pointers, if they were set to a fake parent.
-  if (Archive::is_loading::value && parent == NULL)
   {
     for (size_t i = 0; i < children.size(); ++i)
-    {
-      children[i]->dataset = this->dataset;
       children[i]->parent = this;
-    }
   }
 }
 
@@ -740,8 +701,8 @@ void Octree<MetricType, StatisticType, MatType>::SplitNode(
     // children of index 2^(d - 1) - 1 and less will be on the left side, and
     // all points belonging to children of index 2^(d - 1) and above will be on
     // the right side.
-    SplitInfo s(d, center);
-    const size_t firstRight = split::PerformSplit<MatType, SplitInfo>(*dataset,
+    typename SplitType::SplitInfo s(d, center);
+    const size_t firstRight = split::PerformSplit<MatType, SplitType>(*dataset,
         childBegin, childCount, s);
 
     // We can set the first index of the right child.  The first index of the
@@ -842,8 +803,8 @@ void Octree<MetricType, StatisticType, MatType>::SplitNode(
     // children of index 2^(d - 1) - 1 and less will be on the left side, and
     // all points belonging to children of index 2^(d - 1) and above will be on
     // the right side.
-    SplitInfo s(d, center);
-    const size_t firstRight = split::PerformSplit<MatType, SplitInfo>(*dataset,
+    typename SplitType::SplitInfo s(d, center);
+    const size_t firstRight = split::PerformSplit<MatType, SplitType>(*dataset,
         childBegin, childCount, s, oldFromNew);
 
     // We can set the first index of the right child.  The first index of the
