@@ -14,6 +14,8 @@
 // In case it wasn't included already for some reason.
 #include "spill_tree.hpp"
 
+#include <queue>
+
 namespace mlpack {
 namespace tree {
 
@@ -228,7 +230,7 @@ SpillTree(SpillTree&& other) :
   other.dataset = NULL;
   other.localDataset = false;
 
-  //Set new parent.
+  // Set new parent.
   if (left)
     left->parent = this;
   if (right)
@@ -248,12 +250,12 @@ template<typename Archive>
 SpillTree<MetricType, StatisticType, MatType, HyperplaneType, SplitType>::
 SpillTree(
     Archive& ar,
-    const typename boost::enable_if<typename Archive::is_loading>::type*) :
+    const typename std::enable_if_t<Archive::is_loading::value>*) :
     SpillTree() // Create an empty SpillTree.
 {
   // We've delegated to the constructor which gives us an empty tree, and now we
   // can serialize from it.
-  ar >> data::CreateNVP(*this, "tree");
+  ar >> BOOST_SERIALIZATION_NVP(*this);
 }
 
 /**
@@ -327,7 +329,7 @@ template<typename VecType>
 size_t SpillTree<MetricType, StatisticType, MatType, HyperplaneType,
     SplitType>::GetNearestChild(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType> >::type*)
+    typename std::enable_if_t<IsVector<VecType>::value>*)
 {
   if (IsLeaf() || !left || !right)
     return 0;
@@ -353,7 +355,7 @@ template<typename VecType>
 size_t SpillTree<MetricType, StatisticType, MatType, HyperplaneType,
     SplitType>::GetFurthestChild(
     const VecType& point,
-    typename boost::enable_if<IsVector<VecType> >::type*)
+    typename std::enable_if_t<IsVector<VecType>::value>*)
 {
   if (IsLeaf() || !left || !right)
     return 0;
@@ -669,8 +671,8 @@ bool SpillTree<MetricType, StatisticType, MatType, HyperplaneType, SplitType>::
     }
   }
 
-  const double p1 = double (left + rightFrontier) / points.n_elem;
-  const double p2 = double (right + leftFrontier) / points.n_elem;
+  const double p1 = (double) (left + rightFrontier) / points.n_elem;
+  const double p2 = (double) (right + leftFrontier) / points.n_elem;
 
   if ((p1 <= rho || rightFrontier == 0) &&
       (p2 <= rho || leftFrontier == 0))
@@ -743,10 +745,8 @@ template<typename MetricType,
              class SplitType>
 template<typename Archive>
 void SpillTree<MetricType, StatisticType, MatType, HyperplaneType, SplitType>::
-    Serialize(Archive& ar, const unsigned int /* version */)
+    serialize(Archive& ar, const unsigned int /* version */)
 {
-  using data::CreateNVP;
-
   // If we're loading, and we have children, they need to be deleted.
   if (Archive::is_loading::value)
   {
@@ -756,63 +756,49 @@ void SpillTree<MetricType, StatisticType, MatType, HyperplaneType, SplitType>::
       delete right;
     if (!parent && localDataset)
       delete dataset;
+
+    parent = NULL;
+    left = NULL;
+    right = NULL;
   }
 
-  ar & CreateNVP(parent, "parent");
-  ar & CreateNVP(count, "count");
-  ar & CreateNVP(pointsIndex, "pointsIndex");
-  ar & CreateNVP(overlappingNode, "overlappingNode");
-  ar & CreateNVP(hyperplane, "hyperplane");
-  ar & CreateNVP(bound, "bound");
-  ar & CreateNVP(stat, "statistic");
-  ar & CreateNVP(parentDistance, "parentDistance");
-  ar & CreateNVP(furthestDescendantDistance, "furthestDescendantDistance");
-  ar & CreateNVP(dataset, "dataset");
+  ar & BOOST_SERIALIZATION_NVP(count);
+  ar & BOOST_SERIALIZATION_NVP(pointsIndex);
+  ar & BOOST_SERIALIZATION_NVP(overlappingNode);
+  ar & BOOST_SERIALIZATION_NVP(hyperplane);
+  ar & BOOST_SERIALIZATION_NVP(bound);
+  ar & BOOST_SERIALIZATION_NVP(stat);
+  ar & BOOST_SERIALIZATION_NVP(parent);
+  ar & BOOST_SERIALIZATION_NVP(parentDistance);
+  ar & BOOST_SERIALIZATION_NVP(furthestDescendantDistance);
+  ar & BOOST_SERIALIZATION_NVP(dataset);
 
-  if (Archive::is_loading::value && parent == NULL)
+  if (Archive::is_loading::value)
     localDataset = true;
 
   // Save children last; otherwise boost::serialization gets confused.
-  ar & CreateNVP(left, "left");
-  ar & CreateNVP(right, "right");
+  bool hasLeft = (left != NULL);
+  bool hasRight = (right != NULL);
+
+  ar & BOOST_SERIALIZATION_NVP(hasLeft);
+  ar & BOOST_SERIALIZATION_NVP(hasRight);
+
+  if (hasLeft)
+    ar & BOOST_SERIALIZATION_NVP(left);
+  if (hasRight)
+    ar & BOOST_SERIALIZATION_NVP(right);
 
-  // Due to quirks of boost::serialization, if a tree is saved as an object and
-  // not a pointer, the first level of the tree will be duplicated on load.
-  // Therefore, if we are the root of the tree, then we need to make sure our
-  // children's parent links are correct, and delete the duplicated node if
-  // necessary.
   if (Archive::is_loading::value)
   {
-    // Get parents of left and right children, or, NULL, if they don't exist.
-    SpillTree* leftParent = left ? left->Parent() : NULL;
-    SpillTree* rightParent = right ? right->Parent() : NULL;
-
-    // Reassign parent links if necessary.
-    if (left && left->Parent() != this)
-      left->Parent() = this;
-    if (right && right->Parent() != this)
-      right->Parent() = this;
-
-    // Do we need to delete the left parent?
-    if (leftParent != NULL && leftParent != this)
+    if (left)
     {
-      // Sever the duplicate parent's children.  Ensure we don't delete the
-      // dataset, by faking the duplicated parent's parent (that is, we need to
-      // set the parent to something non-NULL; 'this' works).
-      leftParent->Parent() = this;
-      leftParent->Left() = NULL;
-      leftParent->Right() = NULL;
-      delete leftParent;
+      left->parent = this;
+      left->localDataset = false;
     }
-
-    // Do we need to delete the right parent?
-    if (rightParent != NULL && rightParent != this && rightParent != leftParent)
+    if (right)
     {
-      // Sever the duplicate parent's children, in the same way as above.
-      rightParent->Parent() = this;
-      rightParent->Left() = NULL;
-      rightParent->Right() = NULL;
-      delete rightParent;
+      right->parent = this;
+      right->localDataset = false;
     }
   }
 }
