@@ -1,10 +1,12 @@
 /**
  * @file naive_kmeans_impl.hpp
  * @author Ryan Curtin
+ * @author Shikhar Bhardwaj
  *
  * An implementation of a naively-implemented step of the Lloyd algorithm for
- * k-means clustering.  This may still be the best choice for small datasets or
- * datasets with very high dimensionality.
+ * k-means clustering, using OpenMP for parallelization over multiple threads.
+ * This may still be the best choice for small datasets or datasets with very
+ * high dimensionality.
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
@@ -38,28 +40,44 @@ double NaiveKMeans<MetricType, MatType>::Iterate(const arma::mat& centroids,
   counts.zeros(centroids.n_cols);
 
   // Find the closest centroid to each point and update the new centroids.
-  for (size_t i = 0; i < dataset.n_cols; i++)
+  // Computed in parallel over the complete dataset
+  #pragma omp parallel
   {
-    // Find the closest centroid to this point.
-    double minDistance = std::numeric_limits<double>::infinity();
-    size_t closestCluster = centroids.n_cols; // Invalid value.
+    // The current state of the K-means is private for each thread
+    arma::mat localCentroids(centroids.n_rows, centroids.n_cols,
+        arma::fill::zeros);
+    arma::Col<size_t> localCounts(centroids.n_cols, arma::fill::zeros);
 
-    for (size_t j = 0; j < centroids.n_cols; j++)
+    #pragma omp for
+    for (omp_size_t i = 0; i < (omp_size_t) dataset.n_cols; ++i)
     {
-      const double distance = metric.Evaluate(dataset.col(i), centroids.col(j));
+      // Find the closest centroid to this point.
+      double minDistance = std::numeric_limits<double>::infinity();
+      size_t closestCluster = centroids.n_cols; // Invalid value.
 
-      if (distance < minDistance)
+      for (size_t j = 0; j < centroids.n_cols; j++)
       {
-        minDistance = distance;
-        closestCluster = j;
+        const double distance = metric.Evaluate(dataset.col(i),
+            centroids.unsafe_col(j));
+        if (distance < minDistance)
+        {
+          minDistance = distance;
+          closestCluster = j;
+        }
       }
+
+      Log::Assert(closestCluster != centroids.n_cols);
+
+      // We now have the minimum distance centroid index.  Update that centroid.
+      localCentroids.unsafe_col(closestCluster) += dataset.col(i);
+      localCounts(closestCluster)++;
+    }
+    // Combine calculated state from each thread
+    #pragma omp critical
+    {
+      newCentroids += localCentroids;
+      counts += localCounts;
     }
-
-    Log::Assert(closestCluster != centroids.n_cols);
-
-    // We now have the minimum distance centroid index.  Update that centroid.
-    newCentroids.col(closestCluster) += arma::vec(dataset.col(i));
-    counts(closestCluster)++;
   }
 
   // Now normalize the centroid.
