@@ -17,25 +17,34 @@ using namespace mlpack;
 using namespace mlpack::regression;
 
 LinearRegression::LinearRegression(const arma::mat& predictors,
-                                   const arma::vec& responses,
+                                   const arma::rowvec& responses,
                                    const double lambda,
-                                   const bool intercept,
-                                   const arma::vec& weights) :
+                                   const bool intercept) :
+    LinearRegression(predictors, responses, arma::rowvec(), lambda, intercept)
+{}
+
+LinearRegression::LinearRegression(const arma::mat& predictors,
+                                   const arma::rowvec& responses,
+                                   const arma::rowvec& weights,
+                                   const double lambda,
+                                   const bool intercept) :
     lambda(lambda),
     intercept(intercept)
 {
-  Train(predictors, responses, intercept, weights);
+  Train(predictors, responses, weights, intercept);
 }
 
-LinearRegression::LinearRegression(const LinearRegression& linearRegression) :
-    parameters(linearRegression.parameters),
-    lambda(linearRegression.lambda)
-{ /* Nothing to do. */ }
+void LinearRegression::Train(const arma::mat& predictors,
+                             const arma::rowvec& responses,
+                             const bool intercept)
+{
+  Train(predictors, responses, arma::rowvec(), intercept);
+}
 
 void LinearRegression::Train(const arma::mat& predictors,
-                             const arma::vec& responses,
-                             const bool intercept,
-                             const arma::vec& weights)
+                             const arma::rowvec& responses,
+                             const arma::rowvec& weights,
+                             const bool intercept)
 {
   this->intercept = intercept;
 
@@ -51,14 +60,14 @@ void LinearRegression::Train(const arma::mat& predictors,
   const size_t nCols = predictors.n_cols;
 
   arma::mat p = predictors;
-  arma::vec r = responses;
+  arma::rowvec r = responses;
 
   // Here we add the row of ones to the predictors.
   // The intercept is not penalized. Add an "all ones" row to design and set
   // intercept = false to get a penalized intercept.
   if (intercept)
   {
-    p.insert_rows(0, arma::ones<arma::mat>(1,nCols));
+    p.insert_rows(0, arma::ones<arma::mat>(1, nCols));
   }
 
   if (weights.n_elem > 0)
@@ -88,19 +97,17 @@ void LinearRegression::Train(const arma::mat& predictors,
   // B = Q^T * responses * R^-1
   // If lambda > 0, then we must add a bunch of empty responses.
   if (lambda == 0.0)
-  {
-    arma::solve(parameters, R, arma::trans(Q) * r);
-  }
+    arma::solve(parameters, R, arma::trans(r * Q));
   else
   {
     // Copy responses into larger vector.
-    r.insert_rows(nCols,p.n_cols - nCols);
-    arma::solve(parameters, R, arma::trans(Q) * r);
+    r.insert_cols(nCols, p.n_cols - nCols);
+    arma::solve(parameters, R, arma::trans(r * Q));
   }
 }
 
-void LinearRegression::Predict(const arma::mat& points, arma::vec& predictions)
-    const
+void LinearRegression::Predict(const arma::mat& points,
+    arma::rowvec& predictions) const
 {
   if (intercept)
   {
@@ -109,23 +116,22 @@ void LinearRegression::Predict(const arma::mat& points, arma::vec& predictions)
     Log::Assert(points.n_rows == parameters.n_rows - 1);
     // Get the predictions, but this ignores the intercept value
     // (parameters[0]).
-    predictions = arma::trans(arma::trans(parameters.subvec(1,
-        parameters.n_elem - 1)) * points);
+    predictions = arma::trans(parameters.subvec(1, parameters.n_elem - 1))
+        * points;
     // Now add the intercept.
     predictions += parameters(0);
   }
   else
   {
-    // We want to be sure we have the correct number of dimensions in the dataset.
+    // We want to be sure we have the correct number of dimensions in
+    // the dataset.
     Log::Assert(points.n_rows == parameters.n_rows);
-    predictions = arma::trans(arma::trans(parameters) * points);
+    predictions = arma::trans(parameters) * points;
   }
-
 }
 
-//! Compute the L2 squared error on the given predictors and responses.
 double LinearRegression::ComputeError(const arma::mat& predictors,
-                                      const arma::vec& responses) const
+                                      const arma::rowvec& responses) const
 {
   // Get the number of columns and rows of the dataset.
   const size_t nCols = predictors.n_cols;
@@ -133,7 +139,7 @@ double LinearRegression::ComputeError(const arma::mat& predictors,
 
   // Calculate the differences between actual responses and predicted responses.
   // We must also add the intercept (parameters(0)) to the predictions.
-  arma::vec temp;
+  arma::rowvec temp;
   if (intercept)
   {
     // Ensure that we have the correct number of dimensions in the dataset.
@@ -142,8 +148,8 @@ double LinearRegression::ComputeError(const arma::mat& predictors,
       Log::Fatal << "The test data must have the same number of columns as the "
           "training file." << std::endl;
     }
-    temp = responses - arma::trans( (arma::trans(parameters.subvec(1,
-        parameters.n_elem - 1)) * predictors) + parameters(0));
+    temp = responses - (parameters(0) +
+        arma::trans(parameters.subvec(1, parameters.n_elem - 1)) * predictors);
   }
   else
   {
@@ -153,7 +159,7 @@ double LinearRegression::ComputeError(const arma::mat& predictors,
       Log::Fatal << "The test data must have the same number of columns as the "
           "training file." << std::endl;
     }
-    temp = responses - arma::trans((arma::trans(parameters) * predictors));
+    temp = responses - arma::trans(parameters) * predictors;
   }
   const double cost = arma::dot(temp, temp) / nCols;
 
