@@ -1,6 +1,7 @@
 /**
  * @file logistic_regression.hpp
  * @author Sumedh Ghaisas
+ * @author Arun Reddy
  *
  * The LogisticRegression class, which implements logistic regression.  This
  * implements supports L2-regularization.
@@ -28,6 +29,10 @@ namespace regression {
  * parameter; for instance, logistic regression can be performed on sparse
  * datasets by specifying arma::sp_mat as the MatType parameter.
  *
+ * LogisticRegression can be used for general classification tasks, but the
+ * class is restricted to support only two classes.  For multiclass logistic
+ * regression, see mlpack::regression::SoftmaxRegression.
+ *
  * @tparam MatType Type of data matrix.
  */
 template<typename MatType = arma::mat>
@@ -71,7 +76,7 @@ class LogisticRegression
    */
   LogisticRegression(const MatType& predictors,
                      const arma::Row<size_t>& responses,
-                     const arma::vec& initialPoint,
+                     const arma::rowvec& initialPoint,
                      const double lambda = 0);
 
   /**
@@ -84,7 +89,7 @@ class LogisticRegression
    * @param dimensionality Dimensionality of the data.
    * @param lambda L2-regularization parameter.
    */
-  LogisticRegression(const size_t dimensionality,
+  LogisticRegression(const size_t dimensionality = 0,
                      const double lambda = 0);
 
   /**
@@ -92,15 +97,21 @@ class LogisticRegression
    * data.  This will train the model.  This overload takes an already
    * instantiated optimizer (which holds the LogisticRegressionFunction error
    * function, which must also be instantiated), so that the optimizer can be
-   * configured before the training is run by this constructor.  The predictors
-   * and responses and initial point are all taken from the error function
-   * contained in the optimizer.
+   * configured before the training is run by this constructor.  The update
+   * policy of the optimizer can be set through the policy argument.  The
+   * predictors and responses and initial point are all taken from the error
+   * function contained in the optimizer.
    *
+   * @param predictors Input training variables.
+   * @param responses Outputs results from input training variables.
    * @param optimizer Instantiated optimizer with instantiated error function.
+   * @param lambda L2-regularization parameter.
    */
-  template<template<typename> class OptimizerType>
-  LogisticRegression(
-      OptimizerType<LogisticRegressionFunction<MatType>>& optimizer);
+  template<typename OptimizerType>
+  LogisticRegression(const MatType& predictors,
+                     const arma::Row<size_t>& responses,
+                     OptimizerType& optimizer,
+                     const double lambda);
 
   /**
    * Train the LogisticRegression model on the given input data.  By default,
@@ -115,9 +126,7 @@ class LogisticRegression
    * @param predictors Input training variables.
    * @param responses Outputs results from input training variables.
    */
-  template<
-      template<typename> class OptimizerType = mlpack::optimization::L_BFGS
-  >
+  template<typename OptimizerType = mlpack::optimization::L_BFGS>
   void Train(const MatType& predictors,
              const arma::Row<size_t>& responses);
 
@@ -133,40 +142,25 @@ class LogisticRegression
    * optimizer.Function().GetInitialPoint() to the current parameters vector,
    * accessible via Parameters().
    *
+   * @param predictors Input training variables.
+   * @param responses Outputs results from input training variables.
    * @param optimizer Instantiated optimizer with instantiated error function.
    */
-  template<
-      template<typename> class OptimizerType = mlpack::optimization::L_BFGS
-  >
-  void Train(OptimizerType<LogisticRegressionFunction<MatType>>& optimizer);
+  template<typename OptimizerType>
+  void Train(const MatType& predictors,
+             const arma::Row<size_t>& responses,
+             OptimizerType& optimizer);
 
   //! Return the parameters (the b vector).
-  const arma::vec& Parameters() const { return parameters; }
+  const arma::rowvec& Parameters() const { return parameters; }
   //! Modify the parameters (the b vector).
-  arma::vec& Parameters() { return parameters; }
+  arma::rowvec& Parameters() { return parameters; }
 
   //! Return the lambda value for L2-regularization.
   const double& Lambda() const { return lambda; }
   //! Modify the lambda value for L2-regularization.
   double& Lambda() { return lambda; }
 
-  /**
-   * Predict the responses to a given set of predictors.  The responses will be
-   * either 0 or 1.  Optionally, specify the decision boundary; logistic
-   * regression returns a value between 0 and 1.  If the value is greater than
-   * the decision boundary, the response is taken to be 1; otherwise, it is 0.
-   * By default the decision boundary is 0.5.
-   *
-   * This method is deprecated---you should use Classify() instead.
-   *
-   * @param predictors Input predictors.
-   * @param responses Vector to put output predictions of responses into.
-   * @param decisionBoundary Decision boundary (default 0.5).
-   */
-  mlpack_deprecated void Predict(const MatType& predictors,
-                                 arma::Row<size_t>& responses,
-                                 const double decisionBoundary = 0.5) const;
-
   /**
    * Classify the given point.  The predicted label is returned.  Optionally,
    * specify the decision boundary; logistic regression returns a value between
@@ -237,11 +231,11 @@ class LogisticRegression
 
   //! Serialize the model.
   template<typename Archive>
-  void Serialize(Archive& ar, const unsigned int /* version */);
+  void serialize(Archive& ar, const unsigned int /* version */);
 
  private:
   //! Vector of trained parameters (size: dimensionality plus one).
-  arma::vec parameters;
+  arma::rowvec parameters;
   //! L2-regularization penalty parameter.
   double lambda;
 };
