@@ -359,7 +359,7 @@ BOOST_AUTO_TEST_CASE(SimpleCVWithDTTest)
 
   arma::rowvec weights(4000, arma::fill::randu);
 
-  size_t numClasses = 3;
+  size_t numClasses = 5;
   size_t minimumLeafSize = 8;
 
   {
@@ -402,8 +402,8 @@ BOOST_AUTO_TEST_CASE(KFoldCVMSETest)
   arma::mat data("0 1  0 1");
   arma::rowvec responses("0 1  1 3");
 
-  // 2-fold cross-validation.
-  KFoldCV<LinearRegression, MSE> cv(2, data, responses);
+  // 2-fold cross-validation, no shuffling.
+  KFoldCV<LinearRegression, MSE> cv(2, data, responses, false);
 
   // In each of two validation tests the MSE value should be the same.
   double expectedMSE =
@@ -427,8 +427,9 @@ BOOST_AUTO_TEST_CASE(KFoldCVAccuracyTest)
   arma::Row<size_t> labels("0 0 0 0 1 1 1 1 1 1");
   size_t numClasses = 2;
 
-  // 10-fold cross-validation.
-  KFoldCV<NaiveBayesClassifier<>, Accuracy> cv(10, data, labels, numClasses);
+  // 10-fold cross-validation, no shuffling.
+  KFoldCV<NaiveBayesClassifier<>, Accuracy> cv(10, data, labels, numClasses,
+      false);
 
   // We should succeed in classifying separately the first nine samples, and
   // fail with the remaining one.
@@ -452,7 +453,8 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithWeightedLRTest)
   arma::rowvec weights("1 1 0 0");
 
   KFoldCV<LinearRegression, MSE> cv(2, arma::join_rows(data, data),
-      arma::join_rows(responses, responses), arma::join_rows(weights, weights));
+      arma::join_rows(responses, responses), arma::join_rows(weights, weights),
+      false);
   cv.Evaluate();
 
   arma::mat testData("3 4");
@@ -483,12 +485,12 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   arma::Row<size_t> doubledLabels = arma::join_rows(labels, labels);
   arma::rowvec doubledWeights = arma::join_rows(weights, weights);
 
-  size_t numClasses = 3;
+  size_t numClasses = 5;
   size_t minimumLeafSize = 8;
 
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
-        doubledLabels, numClasses);
+        doubledLabels, numClasses, false);
     cv.Evaluate(minimumLeafSize);
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data, labels,
         numClasses, minimumLeafSize);
@@ -497,7 +499,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
-        datasetInfo, doubledLabels, numClasses);
+        datasetInfo, doubledLabels, numClasses, false);
     cv.Evaluate(minimumLeafSize);
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data,
         datasetInfo, labels, numClasses, minimumLeafSize);
@@ -506,7 +508,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
-        doubledLabels, numClasses, doubledWeights);
+        doubledLabels, numClasses, doubledWeights, false);
     cv.Evaluate(minimumLeafSize);
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data, labels,
         numClasses, weights, minimumLeafSize);
@@ -515,7 +517,7 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   }
   {
     KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(2, doubledData,
-        datasetInfo, doubledLabels, numClasses, doubledWeights);
+        datasetInfo, doubledLabels, numClasses, doubledWeights, false);
     cv.Evaluate(minimumLeafSize);
     arma::Row<size_t> predictedLabels = PredictLabelsWithDT(data, data,
         datasetInfo, labels, numClasses, weights, minimumLeafSize);
@@ -524,4 +526,154 @@ BOOST_AUTO_TEST_CASE(KFoldCVWithDTTest)
   }
 }
 
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k and no shuffling.
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKNoShuffle)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(5, data,
+      datasetInfo, labels, numClasses, false);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k such that the number of points in each
+ * cross-validation bin is not even (the last is smaller), and also with no
+ * shuffling.
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsNoShuffle)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(7, data, datasetInfo,
+      labels, numClasses, false);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k.
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeK)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(5, data,
+      datasetInfo, labels, numClasses);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k such that the number of points in each
+ * cross-validation bin is not even (the last is smaller).
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBins)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(7, data, datasetInfo,
+      labels, numClasses);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k and weights.
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestLargeKWeighted)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+  arma::rowvec weights(data.n_cols, arma::fill::randu);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(5, data,
+      datasetInfo, labels, numClasses, weights);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
+/**
+ * Test k-fold cross-validation with decision trees constructed in multiple
+ * ways, but with larger k such that the number of points in each
+ * cross-validation bin is not even (the last is smaller) and weights.
+ */
+BOOST_AUTO_TEST_CASE(KFoldCVWithDTTestUnevenBinsWeighted)
+{
+  arma::mat data;
+  arma::Row<size_t> labels;
+  data::DatasetInfo datasetInfo;
+  MockCategoricalData(data, labels, datasetInfo);
+  arma::rowvec weights(data.n_cols, arma::fill::randu);
+
+  size_t numClasses = 5;
+  size_t minimumLeafSize = 5;
+
+  KFoldCV<DecisionTree<InformationGain>, Accuracy> cv(7, data, datasetInfo,
+      labels, numClasses, weights);
+  cv.Evaluate(minimumLeafSize);
+  double accuracy = Accuracy::Evaluate(cv.Model(), data, labels);
+
+  // This is a very loose tolerance, but we expect about the same as we would
+  // from an individual decision tree training.
+  BOOST_REQUIRE_GT(accuracy, 0.7);
+}
+
 BOOST_AUTO_TEST_SUITE_END();
